-- Dynamic History Culture
-- Author: Gedemon
-- DateCreated: 2/1/2011 7:34:01 PM
--------------------------------------------------------------

print("Loading Culture Functions...")
print("-------------------------------------")



-------------------------------------------------
-- Plot Over text update functions
-------------------------------------------------

local tipControls = {}
TTManager:GetTypeControlTable( "HexDetails", tipControls )

local m_iCurrentX = -1
local m_iCurrentY = -1

local m_fTime = 0

function GetCulturePlotHelpString(plotCulture, plotKey)

	table.sort(plotCulture, function(a,b) return a.Value > b.Value end)

	local AddedString = ""

	-- this will load from revolution mod...
	local cultureRelations = LoadData("CultureRelations")
	local bShowRevolutionInfo = false

	local plot = GetPlotFromKey ( plotKey )
	local owner = plot:GetOwner()

	if (owner ~= -1) and (cultureRelations ~= nil) and cultureRelations[owner] then
		bShowRevolutionInfo = true
	end

	local totalCulture = 0
	for i = 1, #plotCulture do
		totalCulture = totalCulture + plotCulture[i].Value
	end
	if (totalCulture > 0) then -- don't mess with the universe
		AddedString = AddedString .. "[NEWLINE]Culture : " .. GetCultureValueString(totalCulture) .. " [ICON_CULTURE]"
		if ( DEBUG_SHOW_PLOT_CULTURE ) then
			AddedString = AddedString .. " (" .. totalCulture .. ") at Plot (" .. plotKey .. ")"
		end
		for i = 1, #plotCulture do
			local cultureID = plotCulture[i].ID
			local civAdj = GetCultureTypeAdj(cultureID)

			AddedString = AddedString .. "[NEWLINE]" .. round(plotCulture[i].Value / totalCulture * 100) .. "%  " .. civAdj

			if bShowRevolutionInfo and cultureRelations[owner][cultureID] then
				AddedString = AddedString .. "  (" .. GetRelationValueString(cultureRelations[owner][cultureID]) .. ")"
			end

		end
	end
	return AddedString
end

-- Cursor UI On Mouse Over Plot
function UpdatePlotHelptext()
	-- Add plot culture UI in Plot Mouse Over box
	local plot = Map.GetPlot( m_iCurrentX, m_iCurrentY );
	if (plot ~= nil) then
		local plotKey = GetPlotKey (plot)
		local cultureMap = LoadCultureMap()
		local plotCulture = GetPlotCulture( plotKey, cultureMap )
		if ( plotCulture ) then
			local AddedString = GetCulturePlotHelpString(plotCulture, plotKey)				
			if ( ContextPtr:LookUpControl("/InGame/PlotHelpManager") ) then
				TextString = tipControls.Text:GetText()
				if ( string.find(TextString, "Culture", 1, true) == nil ) then -- prevent text flooding on world view auto-scrolling
					tipControls.Text:SetText( TextString .. AddedString )
					tipControls.Grid:DoAutoSize();
					ContextPtr:LookUpControl("/InGame/PlotHelpManager/TheBox"):SetToolTipType( "HexDetails" )
				end
			end			
		end
	end
end

function Reset()
	m_fTime = 0
	ContextPtr:LookUpControl("/InGame/PlotHelpManager/TheBox"):SetToolTipType()
end

function ProcessInput( uiMsg, wParam, lParam )
    if( uiMsg == MouseEvents.MouseMove ) then
        x, y = UIManager:GetMouseDelta()
        if( x ~= 0 or y ~= 0 ) then 
			Reset()
        end
    end
end
ContextPtr:SetInputHandler( ProcessInput )

function OnUpdate( fDTime )

	local bHasMouseOver = ContextPtr:LookUpControl("/InGame/PlotHelpManager/TheBox"):HasMouseOver()

	if( not bHasMouseOver ) then
		return
	end

	m_fTime = m_fTime + fDTime;

	if( m_fTime > (OptionsManager.GetTooltip1Seconds() / 100) ) then
		UpdatePlotHelptext()
	end
end
ContextPtr:SetUpdate( OnUpdate )

function DoUpdateXY( hexX, hexY )

	local plot = Map.GetPlot( hexX, hexY )
	
	if (plot ~= nil) then
		m_iCurrentX = hexX
		m_iCurrentY = hexY
	end
	
end
Events.SerialEventMouseOverHex.Add( DoUpdateXY )

-- Minimap UI On Mouse Over Plot
function CultureOnMouseOverHex( hexX, hexY )
	-- Add plot culture UI in PlotHelp box
	local plot = Map.GetPlot( hexX, hexY );
	if (plot ~= nil) then
		local plotKey = GetPlotKey (plot)
		local cultureMap = LoadCultureMap()
		local plotCulture = GetPlotCulture( plotKey, cultureMap )
		if ( plotCulture ) then		
			local AddedString = GetCulturePlotHelpString(plotCulture, plotKey)
			if ( ContextPtr:LookUpControl("/InGame/WorldView/PlotHelpText") ) then
				TextString = ContextPtr:LookUpControl("/InGame/WorldView/PlotHelpText/Text"):GetText()
				ContextPtr:LookUpControl("/InGame/WorldView/PlotHelpText/Text"):SetText( TextString .. AddedString )
				ContextPtr:LookUpControl("/InGame/WorldView/PlotHelpText/TextBox"):DoAutoSize()
			end
		end
	end
end
Events.SerialEventMouseOverHex.Add( CultureOnMouseOverHex )

-------------------------------------------------
-------------------------------------------------

function GetPlotCulture( plotKey, cultureMap )
	-- return a table with all civs culture for a plot in cultureMap
	for key, plotCulture in pairs ( cultureMap ) do
		if (key == plotKey) then
			return plotCulture
		end
	end
	return false
end
 
function GetPlotTotalCulture( plotKey, cultureMap )
	-- return the total culture of a plot
	local bDebugOutput = false
	local totalCulture = 0
	for key, plotCulture in pairs (cultureMap) do
		if (key == plotKey) then
			for i = 1, #plotCulture do
				totalCulture = totalCulture + plotCulture[i].Value
			end
			Dprint("Total culture for ("..plotKey..") is : " .. totalCulture , bDebugOutput )
			Dprint ("---" , bDebugOutput )
			return totalCulture
		end
	end
	Dprint ("---" , bDebugOutput )
	return 0
end

function GetPlotCulturePercent( plotKey, cultureMap )
	-- return a table with civs culture % for a plot in cultureMap and the total culture
	local bDebugOutput = false
	local plotCulturePercent = {}
	local totalCulture = 0
	for key, plotCulture in pairs (cultureMap) do
		if (key == plotKey) then
			for i = 1, #plotCulture do
				totalCulture = totalCulture + plotCulture[i].Value
			end
			Dprint("Total culture for ("..plotKey..") is : " .. totalCulture , bDebugOutput )
			if (totalCulture > 0) then -- don't mess with the universe
				for i = 1, #plotCulture do
					table.insert (plotCulturePercent, { ID = plotCulture[i].ID, Value = (plotCulture[i].Value / totalCulture * 100) } )
					Dprint(" - ".. plotCulture[i].ID .. " have " .. (plotCulture[i].Value / totalCulture * 100) .. "%" , bDebugOutput )
				end
			end			
			Dprint ("---" , bDebugOutput )
			return plotCulturePercent, totalCulture
		end
	end
	Dprint ("---" , bDebugOutput )
	return false, 0
end

function GetCivPlotCulture ( plotKey, cultureMap, cultureID )	-- plotKey returned by GetPlotKey(plot), cultureID is civilization cultural ID 
																-- return the culture value for civilization (cultureID) of a given plot (plotKey)
	local civsCulture = GetPlotCulture(plotKey, cultureMap)
	if ( civsCulture ) then
		for i, culture in ipairs ( civsCulture ) do
			if culture.ID == cultureID then
				return culture.Value
			end
		end
	end
	return false
end

function PlotCultureThreshold( plot )
	if plot:IsMountain() then
		return CULTURE_MOUNTAIN_THRESHOLD
	else
		return CULTURE_THRESHOLD
	end
end

function GetCityCulturalOutput ( city, plotKey, cultureMap )
	local bDebugOutput = false

	Dprint ("-- city at (" .. plotKey.. ")", bDebugOutput)
	local playerID = city:GetOwner() 
	local player = Players [ playerID ]
	local ownerCultureID = GetCivTypeFromPlayer (playerID)
	local population = city:GetPopulation()
	local cityCultureProduction = city:GetJONSCulturePerTurn()
	Dprint (ownerCultureID .. " cityCultureProduction : " .. cityCultureProduction .. ", population : " .. population, bDebugOutput)

	local cityCulturalOutput = {}
	local civsCulture, totalCulture = GetPlotCulturePercent(plotKey, cultureMap) -- or GetPlotCulture(plotKey)
	local maxCulture = (population + cityCultureProduction) * CULTURE_CITY_CAPED_FACTOR

	if (totalCulture > maxCulture ) then
		-- should return negative values here ?
		Dprint (" - Max culture reached : ".. maxCulture , bDebugOutput)
		--for i, culture in ipairs ( civsCulture ) do
		--end
		cityCulturalOutput = false
		 
	elseif ( civsCulture ) then	
		local bOwnerHasCulture = false
		for i, culture in ipairs ( civsCulture ) do
			if culture.Value > 0 then -- just in case...
				if (culture.ID == ownerCultureID) then
				-- add a min value
					value = (population + cityCultureProduction) * math.log10(culture.Value * CULTURE_CITY_FACTOR)
					bOwnerHasCulture = true
				else
					value = population * math.log10(culture.Value * CULTURE_CITY_FACTOR)
				end
				Dprint (" - output value for ".. culture.ID .." = " .. value, bDebugOutput)
				table.insert (cityCulturalOutput, { ID = culture.ID, Value = value } )
			end
		end
		
		if not bOwnerHasCulture then
			Dprint (" - ".. ownerCultureID .." has no culture, adding initial value", bDebugOutput)
			table.insert (cityCulturalOutput, { ID = ownerCultureID, Value = CULTURE_CITY_BASE_PRODUCTION })
		end

	else
		Dprint (" - no culture yet, returning initial value...", bDebugOutput)
		-- to do : remove magic number
		table.insert (cityCulturalOutput, { ID = ownerCultureID, Value = CULTURE_CITY_BASE_PRODUCTION })
	end
	
	Dprint ("------------------ ", bDebugOutput)
	return cityCulturalOutput
end

function UpdatePlotOwnership(cultureMap)

	local bDebugOutput = true

	Dprint ("------------------ ", bDebugOutput)
	Dprint ("Update plot ownership... ", bDebugOutput)

	-- get a dead civ table for checking
	-- I should have used playerID as cultureID, but that would have made this incompatible with my Dynamic History Mod...
	local deadCivTypes = {}
	for playerID = 0, GameDefines.MAX_CIV_PLAYERS - 1 do
		local player = Players[playerID]
		if ( not player:IsAlive() ) then
			local bReportError = false					
			deadType = GetCivTypeFromPlayer (playerID, bReportError)
			if (deadType) then
				Dprint ("   - don't check for : " .. deadType .. ", is dead ", bDebugOutput)
				table.insert (deadCivTypes, deadType )
			end
		end
	end

	local lockedMap = LoadLockedMap()

	for key, plotCulture in pairs ( cultureMap ) do
		if lockedMap[key] and lockedMap[key] > 0 then -- plot is locked, don't try to change ownership...
			Dprint ("   - don't check for : (" .. key .. "), plot is locked", bDebugOutput)
		else
			local x,y = GetPlotXYFromKey (key)
			local plot = GetPlot(x,y)
			if not ( plot:IsCity() ) then
				local OwnerType = ""
				local OwnerValue = 0
				local OwnerID

				for i, culture in ipairs ( plotCulture ) do
					if (culture.Value > OwnerValue) then
						local isDead = false
						for i, type in ipairs(deadCivTypes) do
							if (culture.ID == type) then
								isDead = true
							end
						end
						if not (isDead) then
							OwnerType = culture.ID
							OwnerValue = culture.Value
						end
					end 
				end
			
				if ( OwnerType ~= "" ) then 

					for row in GameInfo.Civilizations() do
						if row.Type == OwnerType then
							OwnerID = GetPlayerIDFromCivID (row.ID, false)
						end
					end
					if not OwnerID then
						for row in GameInfo.MinorCivilizations() do
							if row.Type == OwnerType then
								OwnerID = GetPlayerIDFromCivID (row.ID, true)
							end
						end
					end
					-- if the civ with the most culture is different than the present owner and tile flipping is allowed and the culture value is enough
					-- or the tile was not owned and the culture value is enough then change tile owner...
					local bReportError = false
					local ancientOwner = GetCivTypeFromPlayer(plot:GetOwner(), bReportError) or "none"
					local ancientValue =  GetCivPlotCulture ( key, cultureMap, ancientOwner ) or 0
					local totalCulture = GetPlotTotalCulture ( key, cultureMap )
					if ( ancientOwner ~= "none" and OwnerType ~=  ancientOwner and ALLOW_TILE_FLIPPING and OwnerValue > CULTURE_MINIMUM_OWNER and OwnerValue > totalCulture*CULTURE_FLIPPING_RATIO)
						or ( ancientOwner == "none" and ALLOW_CIV4_SPREADING and OwnerValue > CULTURE_MINIMUM_OWNER ) then
						local bReportError = false
						local city = GetCloseCity ( OwnerID, plot )
						Dprint ("------------------ ", bDebugOutput)
						if city ~= nil and (Map.PlotDistance(city:GetX(), city:GetY(), plot:GetX(), plot:GetY()) <= CULTURE_FLIPPING_MAX_DISTANCE) then
							Dprint(" - ownership changed from " .. ancientOwner .. "(" .. ancientValue .. ") to " .. OwnerType .. "(".. OwnerValue .. ") at (" .. key .. ") => " .. city:GetName() , bDebugOutput)
							local player = Players[OwnerID]
							if ancientOwner ~= "none" then
								local ancientPlayer =  Players[plot:GetOwner()]
								player:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, player:GetName() .. " has annexed a part of ".. ancientPlayer:GetName() .." territory with support of local culture group near ".. city:GetName(), player:GetName() .. "  has annexed ".. ancientPlayer:GetName() .." territory !", plot:GetX(), plot:GetY())
								ancientPlayer:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, player:GetName() .. " has annexed a part of ".. ancientPlayer:GetName() .." territory with support of local culture group near ".. city:GetName(), player:GetName() .. "  has annexed ".. ancientPlayer:GetName() .." territory !", plot:GetX(), plot:GetY())
							else
								player:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, player:GetName() .. " has acquired new territory with support of local culture group near ".. city:GetName(), player:GetName() .. "  has acquired new territory !", plot:GetX(), plot:GetY())
							end
							plot:SetOwner(OwnerID, city:GetID())
						else
						
							Dprint(" - WARNING : ownership can't be changed from " .. ancientOwner .. "(" .. ancientValue .. ") to " .. OwnerType .. "(".. OwnerValue .. ") at (" .. key .. ") : can't found close city" , bDebugOutput)
						end	
					end 
				end
			end
		end
	end
	Dprint ("------------------ ", bDebugOutput)
end

-------------------------------------------------
-- Culture Map change/return functions 
-- use : cultureMap = function()
-------------------------------------------------
function ChangeCivPlotCulture ( plotKey, cultureMap, cultureID, value ) -- plotKey: returned by GetPlotKey(plot), cultureID: civ culture ID , value: change to apply
																		-- change a civ culture on a plot by value
																		-- todo : handle negative result
	local bDebugOutput = false

	local civsCulture = GetPlotCulture( plotKey, cultureMap )
	if ( civsCulture ) then
		for i, culture in ipairs ( civsCulture ) do
			if (culture.ID == cultureID) then
				Dprint (" - changing culture value of " .. cultureID .. " at " .. plotKey, bDebugOutput)
				cultureMap[plotKey][i].Value = math.floor(culture.Value + value)
				return cultureMap
			end
		end
		-- no entry for this civ, add it
		Dprint (" - first entry for " .. cultureID .. " at " .. plotKey, bDebugOutput)
		table.insert (cultureMap[plotKey], { ID = cultureID, Value = math.floor(value) } )
		return cultureMap
	else
		-- no entry for this plot, add it
		Dprint (" - plot first entry at (" .. plotKey .. "), first in are " .. cultureID, bDebugOutput)		
		cultureMap[plotKey] = { { ID = cultureID, Value = math.floor(value) } }
		return cultureMap
	end
end

function DecayCulture ( plotKey, cultureMap )

	local bDebugOutput = false
	local plot = GetPlotFromKey(plotKey)
	local ownerID = plot:GetOwner()
	Dprint ("-- Decaying culture at (" .. plotKey.. ")", bDebugOutput)
	local civsCulture = GetPlotCulture( plotKey, cultureMap )
	if ( civsCulture ) then
		for i, culture in ipairs ( civsCulture ) do
			local cultureDecay = (culture.Value * CULTURE_DECAY_RATE/100) + 1
			if (culture.Value - cultureDecay) < 0 then
				if ownerID ~= -1 and culture.ID == GetCivTypeFromPlayer(ownerID) then 
					Dprint (" - " .. cultureMap[plotKey][i].ID .. " is at minimal level of culture on own plot : ".. MINIMAL_CULTURE_ON_OWNED_PLOT , bDebugOutput)
					cultureMap[plotKey][i].Value = MINIMAL_CULTURE_ON_OWNED_PLOT
				else
					Dprint (" - removing " .. cultureMap[plotKey][i].ID , bDebugOutput)
					table.remove ( cultureMap[plotKey], i )
				end
			else
				cultureMap[plotKey][i].Value = math.floor(culture.Value - cultureDecay)
				Dprint (" - " .. cultureMap[plotKey][i].ID .. " lost ".. cultureDecay ..", new value = " .. cultureMap[plotKey][i].Value , bDebugOutput)
			end
		end
	end
	Dprint ("------------------ ", bDebugOutput)
	return cultureMap
end

function DiffuseCulture ( plot, cultureMap, cultureID, value )

	local bDebugOutput = false

	local x = plot:GetX()
	local y = plot:GetY()

	-- debuging around Paris on YnAEMP giant map...
	--if ( x > 11 and x < 15 and y > 63 and y < 67 ) then bDebugOutput = true; end

	if ( value > PlotCultureThreshold(plot) ) then
		Dprint ("Culture diffusion at ("..x..","..y..") of " .. value .. " point from ".. cultureID, bDebugOutput) 
		Dprint ("-----------------", bDebugOutput) 

		local direction_types = {
			DirectionTypes.DIRECTION_NORTHEAST,
			DirectionTypes.DIRECTION_EAST,
			DirectionTypes.DIRECTION_SOUTHEAST,
			DirectionTypes.DIRECTION_SOUTHWEST,
			DirectionTypes.DIRECTION_WEST,
			DirectionTypes.DIRECTION_NORTHWEST
			}

		for loop, direction in ipairs(direction_types) do
			local adjPlot = Map.PlotDirection( x, y, direction)
			if ( adjPlot ~= nil ) then

				local adjKey = GetPlotKey ( adjPlot )

				-- default values
				local bonus = 0
				local malus = 0
				local diffusion = CULTURE_DIFFUSION -- value diffusion per mil
				local plotMax = value * (CULTURE_NORMAL_MAX / 100)

				-- no difusion on lake, sea, ocean 
				if ( adjPlot:IsWater() ) then
					Dprint ("Culture as drown to " .. adjKey, bDebugOutput)
					diffusion = 0
					plotMax = 0
				else
					-- bonus : following road 
					if ( plot:IsRoute() and adjPlot:IsRoute() ) then -- to do : check for river/bridge, but need to call team, so player and all that, CPU intensive in this boucle ?
						Dprint ("Following road to " .. adjKey, bDebugOutput)
						bonus = bonus + CULTURE_FOLLOW_ROAD_BONUS
						plotMax = plotMax * (CULTURE_FOLLOW_ROAD_MAX / 100) 
					end

					-- bonus : following a river 
					if plot:IsRiverConnection(direction) and not plot:IsRiverCrossing(direction) then
						Dprint ("Following river to " .. adjKey, bDebugOutput)
						bonus = bonus + CULTURE_FOLLOW_RIVER_BONUS
						plotMax = plotMax * (CULTURE_FOLLOW_RIVER_MAX / 100)
					end

					-- malus : crossing a tundra
					if (adjPlot:GetTerrainType() == TerrainTypes.TERRAIN_TUNDRA) then
						Dprint ("Crossing tundra to " .. adjKey, bDebugOutput)
						if value > CULTURE_CROSS_TUNDRA_THRESHOLD then
							malus = malus +  CULTURE_CROSS_TUNDRA_PENALTY
							plotMax =  plotMax * (CULTURE_CROSS_TUNDRA_MAX / 100) 
						else
							diffusion = 0
							plotMax =  plotMax * (CULTURE_CROSS_TUNDRA_MAX / 100) 
						end
					end

					-- malus : crossing a river
					if plot:IsRiverCrossing(direction) then
						Dprint ("Crossing river to " .. adjKey, bDebugOutput)
						if value > CULTURE_CROSS_RIVER_THRESHOLD then
							malus = malus + CULTURE_CROSS_RIVER_PENALTY
							plotMax = plotMax * (CULTURE_CROSS_RIVER_MAX / 100) 
						else
							diffusion = 0
							plotMax = plotMax * (CULTURE_CROSS_RIVER_MAX / 100) 
						end
					end
				
					-- malus : crossing a desert
					if (adjPlot:GetTerrainType() == TerrainTypes.TERRAIN_DESERT) then
						Dprint ("Crossing desert to " .. adjKey, bDebugOutput)
						if value > CULTURE_CROSS_DESERT_THRESHOLD then
							malus = malus + CULTURE_CROSS_DESERT_PENALTY
							plotMax = plotMax * (CULTURE_CROSS_DESERT_MAX / 100)  
						else
							diffusion = 0
							plotMax = plotMax * (CULTURE_CROSS_DESERT_MAX / 100) 
						end
					end

					-- malus : crossing snow
					if (adjPlot:GetTerrainType() == TerrainTypes.TERRAIN_SNOW) then
						Dprint ("Crossing snow to " .. adjKey, bDebugOutput)
						if value > CULTURE_CROSS_SNOW_THRESHOLD then
							malus = malus + CULTURE_CROSS_SNOW_PENALTY
							plotMax = plotMax * (CULTURE_CROSS_SNOW_MAX / 100) 
						else
							diffusion = 0
							plotMax = plotMax * (CULTURE_CROSS_SNOW_MAX / 100) 
						end
					end

					-- malus : crossing a marsh
					if (adjPlot:GetFeatureType() == FeatureTypes.FEATURE_MARSH) then
						Dprint ("Crossing marsh to " .. adjKey, bDebugOutput)
						if value > CULTURE_CROSS_MARSH_THRESHOLD then
							malus = malus + CULTURE_CROSS_MARSH_PENALTY
							plotMax = plotMax * (CULTURE_CROSS_MARSH_MAX / 100) 
						else
							diffusion = 0
							plotMax = plotMax * (CULTURE_CROSS_MARSH_MAX / 100) 
						end
					end
				
					-- malus : escalading mountain 
					if adjPlot:IsMountain() then
						Dprint ("Escalading mountain to " .. adjKey, bDebugOutput)
						if value > CULTURE_MOUNTAIN_THRESHOLD then
							malus = malus + CULTURE_MOUNTAIN_PENALTY
							plotMax = plotMax * (CULTURE_MOUNTAIN_MAX / 100) 
						else
							diffusion = 0
							plotMax = plotMax * (CULTURE_MOUNTAIN_MAX / 100) 
						end
					end				
 				end

				diffusedCulture = value * ( (diffusion + (diffusion*bonus/100)) / (1000+ (1000*malus/100) ) )
				plotMax =  math.min( plotMax, value*(CULTURE_MAX / 100) )
				
				Dprint ("Total bonus = " .. bonus .. ", total malus = " .. malus, bDebugOutput)

				local prevCulture =	GetCivPlotCulture ( adjKey, cultureMap, cultureID ) or 0
				local nextCulture = prevCulture + diffusedCulture

				if ( nextCulture < plotMax ) and ( diffusedCulture > 0 ) then
					cultureMap = ChangeCivPlotCulture ( adjKey, cultureMap, cultureID, diffusedCulture )
					Dprint (adjKey .. " received " .. diffusedCulture .. " culture, new =" .. nextCulture .. ", (previous =" .. prevCulture .. ", max = " .. plotMax .. ")", bDebugOutput) 
				else
					Dprint (adjKey .. " didn't received any of the " .. diffusedCulture .. " culture (previous =" .. prevCulture .. ", max = " .. plotMax .. ")", bDebugOutput) 
				end
			end
			Dprint ("---", bDebugOutput)
		end		
		Dprint ("-----------------", bDebugOutput) 
	end
	return cultureMap 
end

-------------------------------------------------
-- Main Culture functions
-- Those functions load and save the culture map, don't mix them !
-------------------------------------------------

function CityCultureOnCapture (hexPos, playerID, cityID, newPlayerID)

	local pCityPlot = Map.GetPlot( ToGridFromHex( hexPos.x, hexPos.y ) )
	local cityPlotKey = GetPlotKey(pCityPlot)
	local cultureMap = LoadCultureMap()
	local bDebugOutput = true
	local pCity = pCityPlot:GetPlotCity()
	Dprint ("-- Change city owner and culture after city capture at (" .. cityPlotKey.. ")", bDebugOutput)

	for i = 0, pCity:GetNumCityPlots() - 1, 1 do
		local plot = pCity:GetCityIndexPlot( i )

		-- this loop does not return only the tiles owned by a city as I thought, but all tiles in this city radius
		-- I have not found a way to get only those tiles, so there's another imperfect check here, as it is 
		-- there is more culture removed than it should when city radius (owned by the conqueror) overlap
		-- well let's call it collateral damage... 

		if (plot ~= nil and (plot:GetOwner() == pCity:GetOwner()) ) then
			local plotKey = GetPlotKey(plot)
			local civsCulture = GetPlotCulture( plotKey, cultureMap )
			if ( civsCulture ) then
				local totalCultureLost = 0
				for i, culture in ipairs ( civsCulture ) do
					-- culture lost for all civs
					local cultureLost = (culture.Value * CULTURE_LOST_CONQUEST/100)
					if (culture.Value - cultureLost) < 0 then
						Dprint (" - removing " .. cultureMap[plotKey][i].ID .. " culture at (" .. plotKey.. ")", bDebugOutput)
						table.remove ( cultureMap[plotKey], i ) 
					else
						cultureMap[plotKey][i].Value = math.floor(culture.Value - cultureLost)
						Dprint (" - " .. cultureMap[plotKey][i].ID .. " lost ".. cultureLost .." culture at (" .. plotKey.. "), new value = " .. cultureMap[plotKey][i].Value , bDebugOutput)
					end
					totalCultureLost = totalCultureLost + cultureLost
				end
				-- culture gain for capturing civ
				local cultureGain = totalCultureLost * CULTURE_GAIN_CONQUEST / 100
				local conqueror = GetCivTypeFromPlayer(newPlayerID)
				cultureMap = ChangeCivPlotCulture ( plotKey, cultureMap, conqueror, cultureGain )
				Dprint (" - " .. conqueror .. " gain ".. cultureGain .." culture at (" .. plotKey.. "), new value = " .. GetCivPlotCulture ( plotKey, cultureMap, conqueror ) , bDebugOutput)
			end
		end
	end
	Dprint ("------------------ ", bDebugOutput)
	SaveCultureMap( cultureMap )
end

function UpdateCultureMap()

	-- chrono
	local t_diff1
	local t_diff2
	local t_difft = 0

	local t1 = os.clock()

	Dprint ("Update culture Map")
	Dprint ("------------------")

	local cultureMap = LoadCultureMap()
	local lockedMap = LoadLockedMap()
	local bDebugOutput = false

	local t2 = os.clock()
	-- loop the map plots to add culture from builders ( cities, tribes, great artists) and apply decay
	for iPlotLoop = 0, Map.GetNumPlots()-1, 1 do
		local plot = Map.GetPlotByIndex(iPlotLoop)

		-- no culture on water, don't waste time
		if not ( plot:IsWater() ) then

			local plotKey = GetPlotKey ( plot )
			--local cultureID
			local plotCulture = GetPlotCulture(plotKey, cultureMap)

			-- apply decay and diffuse culture
			if ( plotCulture ) then
				cultureMap = DecayCulture (plotKey, cultureMap)
				t_diff1 = os.clock()
				for i, culture in ipairs ( plotCulture ) do
					cultureMap = DiffuseCulture ( plot, cultureMap, culture.ID, culture.Value )
				end
				t_diff2 = os.clock()
				t_difft = t_difft + (t_diff2-t_diff1)
			end

			-- add culture in city
			if plot:IsCity() then
				local city = plot:GetPlotCity() 
				local cultureTab = GetCityCulturalOutput ( city, plotKey, cultureMap )
				if ( cultureTab ) then
					for i, culturalOutput in ipairs ( cultureTab ) do
						-- to do : cap city culture
						cultureMap = ChangeCivPlotCulture ( plotKey, cultureMap, culturalOutput.ID, culturalOutput.Value )
						Dprint(culturalOutput.Value .. " - city cultural points generated for " .. culturalOutput.ID .. " at (" .. plotKey .. ")" , bDebugOutput)
						Dprint ("------------------ ", bDebugOutput)
					end
				end
			end	
		
			-- while we're here, remove dead people ownership
			if ( ALLOW_CIV4_SPREADING ) then
				local playerID = plot:GetOwner()
				if playerID ~= -1 then
					if ( not Players[playerID]:IsAlive() ) then
						Dprint ("Removing dead ownership at " .. plotKey , bDebugOutput)
						plot:SetOwner(-1, -1)
					end
				end
			end	
		end		
	end   
	local t3 = os.clock()

	-- Update plot ownership
	if ( ALLOW_CIV4_SPREADING or ALLOW_TILE_FLIPPING) then
		UpdatePlotOwnership(cultureMap)
	end

	local t4 = os.clock()
	
	-- Update locked map
	if ( ALLOW_CULTURE_CONQUEST ) then
		for key, counter in pairs (lockedMap) do
			if counter and counter > 0 then
				lockedMap[key] = counter - 1
			end
		end
	end

	local t5 = os.clock()

	SaveCultureMap( cultureMap )
	SaveLockedMap( lockedMap )

	local t6 = os.clock()

	Dprint("Updating Culture execution time : ") 
	Dprint("Load tables      = " .. t2 - t1) 
	Dprint("Add Culture      = " .. t3 - t2) 
	Dprint(" (inc. diffusion = " .. t_difft .." )")
	Dprint("Update ownership = " .. t4 - t3) 
	Dprint("Update LockedMap = " .. t5 - t4) 
	Dprint("Save tables      = " .. t6 - t5) 
	Dprint("Total time       = " .. t6 - t1) 
	Dprint("-------------------------------------")
end

function OnCultureChangePlot(iHexX, iHexY, playerID, bUnknown)

	if playerID ~= -1 then
		local bDebugOutput = false
		local x, y = ToGridFromHex( iHexX, iHexY )
		local plotKey = x..","..y
		local plot = GetPlotFromKey(plotKey)

		if plot and not plot:IsWater() then
			Dprint ("Update culture on plot ("..plotKey..")", bDebugOutput)
			Dprint ("------------------", bDebugOutput)

			local cultureMap = LoadCultureMap()
			local plotCulture = GetPlotCulture(plotKey, cultureMap)
			local cultureID = GetCivTypeFromPlayer(playerID)
			if (not GetCivPlotCulture ( plotKey, cultureMap, cultureID ) ) and MINIMAL_CULTURE_ON_OWNED_PLOT > 0 then
				cultureMap = ChangeCivPlotCulture ( plotKey, cultureMap, cultureID, MINIMAL_CULTURE_ON_OWNED_PLOT )
				SaveCultureMap( cultureMap )
			end
		end
	end
end


-------------------------------------------------
-- Test functions
-------------------------------------------------

function GetCultureValueString(value)
	if value < 50 then
		return "feeble"
	end
	if value < 200 then
		return "low"
	end
	if value < 500 then
		return "sparse"
	end
	if value < 2000 then
		return "average"
	end
	if value < 5000 then
		return "established"
	end
	return "entrenched"
end

function UnitCaptureTile(playerID, UnitID, x, y)

	if not ALLOW_CULTURE_CONQUEST then
		return
	end

	local bDebug = false
	local plot = Map.GetPlot(x,y)
	if (plot == nil) then
		return
	end
	
	if ( plot:IsCity() or plot:IsWater() ) then
		return
	end

	-- check here if an unit can't capture a tile and return
	local player = Players[ playerID ]
	local unit = player:GetUnitByID(UnitID)
	if not ( unit:IsCombatUnit() ) then
		return
	end
	
	local lockedMap = LoadLockedMap()

	local plotKey = GetPlotKey ( plot )
	local ownerID = plot:GetOwner()

	-- If the unit is moving on another player territory...
	if (playerID ~= ownerID and ownerID ~= -1) then
		Dprint("-------------------------------------", bDebug)
		Dprint("Unit moving on tile ("..x..","..y..") is in another civ (id="..ownerID..") territory", bDebug)

		local player2 = Players[ ownerID ]		
		local team = Teams[ player:GetTeam() ]
		local team2 = Teams[ player2:GetTeam() ]

		-- If we are at war with the other player :
		if team:IsAtWar( player2:GetTeam() ) then
			Dprint(" - Unit owner (id="..playerID..") and tile owner (id="..ownerID..") are at war", bDebug)

			
			local unitCultureID = GetCivTypeFromPlayer(playerID)
			local ownerCultureID = GetCivTypeFromPlayer(ownerID)

			local cultureMap = LoadCultureMap()

			local unitCultureValue = GetCivPlotCulture ( plotKey, cultureMap, unitCultureID ) or 0
			local ownerCultureValue = GetCivPlotCulture ( plotKey, cultureMap, ownerCultureID ) or 0
			
			-- test for capture
			if	(unitCultureValue > ownerCultureValue or CULTURE_CONQUEST_EVEN_LOWER)
				and	(unitCultureValue >= MINIMAL_CULTURE_FOR_CONQUEST or CULTURE_CONQUEST_EVEN_NONE)
				then

				Dprint(" - Unit is capturing territory...", bDebug)
				--local capitalCity = player:GetCapitalCity()
				--plot:SetOwner(playerID, capitalCity:GetID() )				
				local closeCity = GetCloseCity ( playerID, plot )					
				if closeCity and (Map.PlotDistance(closeCity:GetX(), closeCity:GetY(), plot:GetX(), plot:GetY()) <= CULTURE_FLIPPING_MAX_DISTANCE) then
					plot:SetOwner(playerID, closeCity:GetID() )

					lockedMap[plotKey] = LOCKED_TURN_ON_CONQUEST

				else
					Dprint(" - But is too far away from own cities to hold it !", bDebug)
				end

			--[[
			-- capturing current owner territory			
			elseif ( ownerID == firstOwner) then
				Dprint(" - Unit is capturing territory", bDebug)
				--local capitalCity = player:GetCapitalCity()
				--plot:SetOwner(playerID, capitalCity:GetID() )
				
				local closeCity = GetCloseCity ( playerID, plot )					
				if closeCity then
					plot:SetOwner(playerID, closeCity:GetID() )
				else
					plot:SetOwner(playerID, -1 )
				end
			else
				-- don't free old owner territory if we're at war !
				local player3 = Players[ firstOwner ]				
				if team:IsAtWar( player3:GetTeam() ) then
					Dprint(" - Unit is capturing territory, old owner civ (id=".. firstOwner ..") is also at war with unit owner", bDebug)
					--local capitalCity = player:GetCapitalCity()
					--plot:SetOwner(playerID, capitalCity:GetID() )	
					plot:SetOwner(playerID, -1 )			
				elseif ( not player3:IsAlive() ) then
					Dprint(" - Unit is capturing territory, old owner civ (id=".. firstOwner ..") is dead", bDebug)
					--local capitalCity = player:GetCapitalCity()
					--plot:SetOwner(playerID, capitalCity:GetID() )	
					local closeCity = GetCloseCity ( playerID, plot )
					if closeCity then
						plot:SetOwner(playerID, closeCity:GetID() )
					else
						plot:SetOwner(playerID, -1 )
					end
				else
				-- liberating old owner territory
					Dprint(" - Unit is liberating this territory belonging to another civ (id=".. firstOwner ..")", bDebug)

					local closeCity = GetCloseCity ( firstOwner, plot )					
					if closeCity then
						plot:SetOwner(firstOwner, closeCity:GetID() )
					else
						plot:SetOwner(firstOwner, -1 )
					end					
					if player3:IsMinorCiv() and not player:IsMinorCiv() then
						player3:ChangeMinorCivFriendshipWithMajor( playerID, LIBERATE_MINOR_TERRITORY_BONUS ) -- give diplo bonus for liberating minor territory
					end
				end --]]
			end
		end
	end
	
	SaveLockedMap( lockedMap )

end
GameEvents.UnitSetXY.Add( UnitCaptureTile )


function GetRelationValueString(value)
	if value < THRESHOLD_EXASPERATED then
		return "[COLOR_NEGATIVE_TEXT]exasperated[ENDCOLOR]"
	end
	if value < THRESHOLD_WOEFUL then
		return "[COLOR_NEGATIVE_TEXT]woeful[ENDCOLOR]"
	end
	if value < THRESHOLD_UNHAPPY then
		return "unhappy"
	end
	if value < THRESHOLD_CONTENT then
		return "content"
	end
	if value < THRESHOLD_HAPPY then
		return "happy"
	end
	if value < THRESHOLD_JOYFUL then
		return "[COLOR_POSITIVE_TEXT]joyful[ENDCOLOR]"
	end
	return "[COLOR_POSITIVE_TEXT]enthusiastic[ENDCOLOR]"
end

function GetCultureTypeAdj(cultureID)
	local civAdj = ""
	for row in GameInfo.Civilizations() do
		if row.Type == cultureID then
			civAdj = Locale.ConvertTextKey (row.Adjective)
		end
	end
	if civAdj == "" then
		for row in GameInfo.MinorCivilizations() do
			if row.Type == cultureID then
				civAdj = Locale.ConvertTextKey (row.Adjective)
			end
		end
	end

	if cultureID == "SEPARATIST" then
		civAdj = "Separatist"
	end
	return civAdj
end