-- Culture Utils
-- Author: Gedemon
-- DateCreated: 1/29/2011 11:52:55 PM
--------------------------------------------------------------

print("Loading Culture Utils Function...")
print("-------------------------------------")

--------------------------------------------------------------
-- Map functions 
--------------------------------------------------------------

--	here (x,y) = (0,0) is bottom left of map in Worldbuilder.
function GetPlot (x,y)
	local plot = Map:GetPlotXY(y,x)
	return plot
end

function GetPlotKey ( plot )
	-- set the key string used in cultureMap
	-- structure : g_CultureMap[plotKey] = { { ID = CIV_CULTURAL_ID, Value = cultureForThisCiv }, }
	local x = plot:GetX()
	local y = plot:GetY()
	local plotKey = x..","..y
	return plotKey
end

-- return the plot refered by the key string
function GetPlotFromKey ( plotKey )
	local pos = string.find(plotKey, ",")
	local x = string.sub(plotKey, 1 , pos -1)
	local y = string.sub(plotKey, pos +1)
	local plot = Map:GetPlotXY(y,x)
	return plot
end

function GetPlotXYFromKey ( plotKey )
	local pos = string.find(plotKey, ",")
	local x = string.sub(plotKey, 1 , pos -1)
	local y = string.sub(plotKey, pos +1)
	return x, y
end

function GetCloseCity ( playerID, plot )
	local pPlayer = Players[playerID]
	local distance = 1000
	local closeCity = nil
	for pCity in pPlayer:Cities() do
		distanceToCity = Map.PlotDistance(pCity:GetX(), pCity:GetY(), plot:GetX(), plot:GetY())
		if ( distanceToCity < distance) then
			distance = distanceToCity
			closeCity = pCity
		end
	end
	return closeCity
end

--[[

Here are the functions related to the culture map

table : CultureMap ["x,y"] = { { ID = CIVILIZATION_TYPE, Value = culturesum }, ... }
plot x,y are converted to a string "x,y" used as key for the table

]]
function LoadCultureMap()
	local pPlayer = Players[PLAYER_SAVE_SLOT]
	local cultureMap = load( pPlayer, "CultureMap" ) or {}
	return cultureMap
end
function SaveCultureMap( cultureMap )
	local pPlayer = Players[PLAYER_SAVE_SLOT]
	save( pPlayer, "CultureMap", cultureMap )
end

-- Locked plots (just capturad by military units, don't flip back immediatly from culture...
function LoadLockedMap()
	local pPlayer = Players[PLAYER_SAVE_SLOT]
	local LockedMap = load( pPlayer, "LockedMap" ) or {}
	return LockedMap
end
function SaveLockedMap( LockedMap )
	local pPlayer = Players[PLAYER_SAVE_SLOT]
	save( pPlayer, "LockedMap", LockedMap )
end



function LoadData(name, defaultValue)
	local startTime = os.clock()
	local pPlayer = Players[PLAYER_SAVE_SLOT]
	local value = load( pPlayer, name ) or defaultValue
	local endTime = os.clock()
	local totalTime = endTime - startTime
	Dprint ("LoadData() used " .. totalTime .. " sec for " .. name, DEBUG_PERFORMANCE)
	Dprint("-------------------------------------", DEBUG_PERFORMANCE)
	return value
end
function SaveData( name, value )
	startTime = os.clock()
	local pPlayer = Players[PLAYER_SAVE_SLOT]
	save( pPlayer, name, value )
	endTime = os.clock()
	totalTime = endTime - startTime
	Dprint ("SaveData() used " .. totalTime .. " sec for " .. name, DEBUG_PERFORMANCE)
	Dprint("-------------------------------------", DEBUG_PERFORMANCE)
end

--------------------------------------------------------------
-- Math functions 
--------------------------------------------------------------

function round(num)
    under = math.floor(num)
    upper = math.floor(num) + 1
    underV = -(under - num)
    upperV = upper - num
    if (upperV > underV) then
        return under
    else
        return upper
    end
end

--------------------------------------------------------------
-- Database functions 
--------------------------------------------------------------

-- return the first playerID using this CivilizationID or MinorcivID
function GetPlayerIDFromCivID (id, bIsMinor, bReportError)
	if ( bIsMinor ) then
		for player_num = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS - 1, 1 do
			local player = Players[player_num]
			if ( id == player:GetMinorCivType() ) then
				return player_num
			end
		end
	else
		for player_num = 0, GameDefines.MAX_MAJOR_CIVS-1 do
			local player = Players[player_num]
			if ( id == player:GetCivilizationType() ) then
				return player_num
			end
		end
	end
	if (id) then 
		Dprint ("WARNING : can't find Player ID for civ ID = " .. id , bReportError) 
	else	
		Dprint ("WARNING : civID is NILL or FALSE", bReportError) 
	end
	return false
end

-- return Civ type ID for playerID
function GetCivIDFromPlayerID (playerID, bReportError)
	if (playerID ~= -1) then
		if playerID <= GameDefines.MAX_MAJOR_CIVS-1 then
			local civID = Players[playerID]:GetCivilizationType()
			if (civID ~= -1) then
				return civID
			else
				Dprint ("WARNING : no major civ for PlayerId = " .. playerID , bReportError) 
				return false
			end
		else 
			local civID = Players[playerID]:GetMinorCivType()
			if (civID ~= -1) then
				return civID
			else
				Dprint ("WARNING : no minor civ for PlayerId = " .. playerID, bReportError) 
				return false
			end
		end
	else
		Dprint ("WARNING : trying to find CivType for PlayerId = -1", bReportError) 
		return false
	end
end

function GetCivTypeFromPlayer (playerID, bReportError)
	if (playerID ~= -1) then
		if playerID <= GameDefines.MAX_MAJOR_CIVS-1 then
			local civID = Players[playerID]:GetCivilizationType()
			if (civID ~= -1) then
				return GameInfo.Civilizations[civID].Type
			else
				Dprint ("WARNING : no major civ for PlayerId = " .. playerID , bReportError) 
				return false
			end
		else 
			local civID = Players[playerID]:GetMinorCivType()
			if (civID ~= -1) then
				return GameInfo.MinorCivilizations[civID].Type
			else
				Dprint ("WARNING : no minor civ for PlayerId = " .. playerID, bReportError) 
				return false
			end
		end
	else
		Dprint ("WARNING : trying to find CivType for PlayerId = -1", bReportError) 
		return false
	end
end

-- update localized text
function SetText ( str, tag )
	-- in case of language change mid-game :
	local query = "UPDATE Language_en_US SET Text = '".. str .."' WHERE Tag = '".. tag .."'"
	for result in DB.Query(query) do
	end
	--Dprint (query)
	-- that's the table used ingame :
	local query = "UPDATE LocalizedText SET Text = '".. str .."' WHERE Tag = '".. tag .."'"
	for result in DB.Query(query) do
	end	
	--Dprint (query)
end

-- return the first playerID using this Civilization or Minorciv type
function GetPlayerIDFromCivType (type, bIsMinor, bReportError)
	if (type) then 
		local civID = nil
		if GameInfo.Civilizations[type] then
			civID = GameInfo.Civilizations[type].ID
			return GetPlayerIDFromCivID (civID, false, bReportError)
		elseif GameInfo.MinorCivilizations[type] then
			civID = GameInfo.MinorCivilizations[type].ID
			return GetPlayerIDFromCivID (civID, true, bReportError)
		end
		Dprint ("WARNING : can't find Player ID for civ Type = " .. type , bReportError) 
	else	
		Dprint ("WARNING : civID is NILL or FALSE", bReportError) 
	end
	return false
end